/*
 * MIT License
 *
 * Copyright (c) 2019 norangebit
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package it.norangeb.algorithms.graph.operations

import it.norangeb.algorithms.graph.UndirectedGraph
import java.util.ArrayDeque

class DepthFirstOrder(graph: UndirectedGraph) {
    private val visited = Array(graph.vertexNumber()) { false }

    private val preOrder: MutableCollection<Int> = ArrayDeque()
    private val postOrder: MutableCollection<Int> = ArrayDeque()

    init {
        (0 until graph.vertexNumber())
            .forEach {
                if (!visited[it])
                    dfs(graph, it)
            }
    }

    private fun dfs(graph: UndirectedGraph, vertex: Int) {
        preOrder.add(vertex)

        visited[vertex] = true

        graph.adjacentVertex(vertex)
            .forEach {
                if (!visited[it])
                    dfs(graph, it)
            }

        postOrder.add(vertex)
    }

    fun preOrder(): Iterable<Int> {
        return preOrder
    }

    fun postOrder(): Iterable<Int> {
        return postOrder
    }

    fun reversePostOrder(): Iterable<Int> {
        return postOrder.reversed()
    }
}